<?php
namespace TrxEffects\Core\Modules\BgCanvas;

use TrxEffects\Plugin;
use TrxEffects\Core\Modules\Module;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class BgCanvasElementor extends Module {

	/**
	 * Class constructor.
	 *
	 * Initializing module.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}


	/**
	 * Init module
	 *
	 * Do init actions of the module
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		// Add module-specific parameters to the params for all Elementor's modules
		add_action( 'elementor/element/before_section_start', array( $this, 'add_params_to_all_elements' ), 10, 3 );
		add_action( 'elementor/widget/before_section_start', array( $this, 'add_params_to_all_elements' ), 10, 3 );

		// Add module-specific data-attributes to the elements layout
		add_action( 'elementor/frontend/element/before_render', array( $this, 'add_data_attributes_to_all_elements' ), 10, 1 );
		add_action( 'elementor/frontend/widget/before_render', array( $this, 'add_data_attributes_to_all_elements' ), 10, 1 );
	}


	/**
	 * Dynamic background parameters for all elements
	 *
	 * Add module-specific parameters to the params for all Elementor's modules
	 *
	 * @hook action 'elementor/element/before_section_start'
	 * @hook action 'elementor/widget/before_section_start'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_params_to_all_elements( $element, $section_id, $args ) {

		if ( ! is_object( $element ) ) return;

		if ( $element->get_name() == 'common' && $section_id == '_section_responsive' ) {
			
			$element->start_controls_section( 'section_trx_bg_canvas', array(
																		'tab' => ! empty( $args['tab']) ? $args['tab'] : \Elementor\Controls_Manager::TAB_ADVANCED,
																		'label' => esc_html__( 'Dynamic Background', 'trx-effects' )
																	) );
			$element->add_control( 'bg_canvas_type', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Breakpoint type', 'trx-effects' ),
				'label_block' => false,
				'options' => apply_filters( 'trx_effects_filter/bg_canvas_effects', array(
					'none'  => esc_html__( 'None', 'trx-effects' ),
					'start' => esc_html__( 'Start', 'trx-effects' ),
					'end'   => esc_html__( 'End', 'trx-effects' ),
				) ),
				'default' => 'none',
			) );
			$element->add_control( 'bg_canvas_id', array(
				'type' => \Elementor\Controls_Manager::TEXT,
				'label' => esc_html__( 'Breakpoint ID', 'trx-effects' ),
				'label_block' => false,
				'condition' => array(
					'bg_canvas_type!' => 'none'
				),
			) );
			$element->add_control( 'bg_canvas_effect', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Background effect', 'trx-effects' ),
				'label_block' => false,
				'options' => apply_filters( 'trx_effects_filter/bg_canvas_effects', array(
					'round' => esc_html__( 'Round', 'trx-effects' ),
					'fade'  => esc_html__( 'Fade', 'trx-effects' ),
				) ),
				'default' => 'round',
				'condition' => array(
					'bg_canvas_type!' => 'none'
				),
			) );
			$element->add_control( 'bg_canvas_color', array(
				'label' => esc_html__( 'Background color', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::COLOR,
				'global' => array(
					'active' => false,
				),
				'default' => '',
				'condition' => array(
					'bg_canvas_type!' => 'none',
				),
			) );
			$element->add_control( 'bg_canvas_size', array(
				'label' => esc_html__( 'Min.size', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => '',
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 0,
						'max' => 300
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'bg_canvas_type!' => 'none',
					'bg_canvas_effect' => 'round',
				),
			) );
			$element->add_control( 'bg_canvas_shift', array(
				'label' => esc_html__( 'Shift', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => '',
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => -100,
						'max' => 100
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'bg_canvas_type!' => 'none'
				),
			) );

			$element->end_controls_section();
		}
	}


	/**
	 * Add "data-trx-effects-bg-canvas-*"  to the wrapper of the element
	 *
	 * Hook render elements and add data parameters to its layout
	 *
	 * @hook action 'elementor/frontend/element/before_render'
	 * @hook action 'elementor/frontend/widget/before_render'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_data_attributes_to_all_elements( $element ) {
		if ( is_object( $element ) ) {
			$settings = $element->get_settings();
			if ( ! empty( $settings['bg_canvas_type'] ) && ! trx_effects_is_off( $settings['bg_canvas_type'] ) ) {
				$element->add_render_attribute( '_wrapper', array(
					'data-trx-effects-bg-canvas-id'     => $settings['bg_canvas_id'],
					'data-trx-effects-bg-canvas-type'   => $settings['bg_canvas_type'],
					'data-trx-effects-bg-canvas-effect' => $settings['bg_canvas_effect'],
					'data-trx-effects-bg-canvas-size'   => $settings['bg_canvas_size']['size'],
					'data-trx-effects-bg-canvas-shift'  => $settings['bg_canvas_shift']['size'],
					'data-trx-effects-bg-canvas-color'  => $settings['bg_canvas_color']
				) );
			}
		}
	}

}
