<?php
namespace TrxEffects\Core\Modules\BgText;

use TrxEffects\Plugin;
use TrxEffects\Core\Modules\Module;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class BgTextElementor extends Module {

	/**
	 * Class constructor.
	 *
	 * Initializing module.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}


	/**
	 * Init module
	 *
	 * Do init actions of the module
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		// Add module-specific parameters to the params of the section
		add_action( 'elementor/element/before_section_start', array( $this, 'add_params_to_section' ), 10, 3 );

		// Add module-specific data-attributes to the section layout
		add_action( 'elementor/frontend/element/before_render', array( $this, 'add_data_attributes_to_section' ), 10, 1 );
		add_action( 'elementor/frontend/section/before_render', array( $this, 'add_data_attributes_to_section' ), 10, 1 );
	}


	/**
	 * Background text parameters for sections
	 *
	 * Add module-specific parameters to the params of sections
	 *
	 * @hook action 'elementor/element/before_section_start'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_params_to_section( $element, $section_id, $args ) {

		if ( ! is_object( $element ) ) return;

		if ( in_array( $element->get_name(), array( 'section' ) ) && $section_id == '_section_responsive' ) {

			// Register controls
			$element->start_controls_section( 'section_trx_bg_text', array(
				'tab' => ! empty( $args['tab'] ) ? $args['tab'] : \Elementor\Controls_Manager::TAB_ADVANCED,
				'label' => esc_html__( 'Background text', 'trx-effects' )
			) );

			$element->add_control( 'bg_text', array(
				'type' => \Elementor\Controls_Manager::TEXT,
				'label' => esc_html__( 'Text', 'trx-effects' ),
				'label_block' => false,
				'default' => ''
			) );

			$element->add_control(
				'bg_text_color',
				array(
					'label' => esc_html__( 'Text color', 'trx-effects' ),
					'label_block' => false,
					'type' => \Elementor\Controls_Manager::COLOR,
					'global' => array(
						'active' => false,
					),
					'default' => '',
					'selectors' => array(
						'{{WRAPPER}} .trx_effects_bg_text_char' => 'color: {{VALUE}};',
					)
				)
			);

			if ( class_exists( '\Elementor\Group_Control_Typography' ) && class_exists( '\Elementor\Core\Schemes\Typography' ) ) {
				$element->add_group_control(
					\Elementor\Group_Control_Typography::get_type(),
					array(
						'name' => 'bg_text_typography',
						'scheme' => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_1,
						'selector' => '{{WRAPPER}} .trx_effects_bg_text_char',
					)
				);
			}

			if ( class_exists( '\Elementor\Group_Control_Text_Shadow' ) ) {
				$element->add_group_control(
					\Elementor\Group_Control_Text_Shadow::get_type(),
					array(
						'name' => 'bg_text_shadow',
						'selector' => '{{WRAPPER}} .trx_effects_bg_text_char',
					)
				);
			}

			$element->add_responsive_control( 'bg_text_top', array(
				'type' => \Elementor\Controls_Manager::SLIDER,
				'label' => esc_html__( 'Top offset', 'trx-effects' ),
				'default' => array(
					'size' => '',
					'unit' => '%'
				),
				'size_units' => array( 'px', 'em', '%' ),
				'range' => array(
					'px' => array(
						'min' => -200,
						'max' => 200
					),
					'em' => array(
						'min' => -100,
						'max' => 100
					),
					'%' => array(
						'min' => -100,
						'max' => 100
					),
				),
				'selectors' => array(
					'{{WRAPPER}} .trx_effects_bg_text_inner' => 'margin-top: {{SIZE}}{{UNIT}}',
				),
			) );

			$element->add_responsive_control( 'bg_text_left', array(
				'type' => \Elementor\Controls_Manager::SLIDER,
				'label' => is_rtl() ? esc_html__( 'Right offset', 'trx-effects' ) : esc_html__( 'Left offset', 'trx-effects' ),
				'default' => array(
					'size' => '',
					'unit' => '%'
				),
				'size_units' => array( 'px', 'em', '%' ),
				'range' => array(
					'px' => array(
						'min' => -200,
						'max' => 200
					),
					'em' => array(
						'min' => -100,
						'max' => 100
					),
					'%' => array(
						'min' => -100,
						'max' => 100
					),
				),
				'selectors' => array(
					'{{WRAPPER}} .trx_effects_bg_text_inner' => is_rtl() ? 'margin-right: {{SIZE}}{{UNIT}};' : 'margin-left: {{SIZE}}{{UNIT}};',
				),
			) );

			$element->add_control( 'bg_text_effect', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Entrance effect', 'trx-effects' ),
				'label_block' => false,
				'options' => apply_filters( 'trx_effects_filter/bg_text_effects', array(
					'none'   => esc_html__( 'None', 'trx-effects' ),
					'rotate' => esc_html__( 'Rotate', 'trx-effects' ),
					'slide'  => esc_html__( 'Slide', 'trx-effects' ),
				) ),
				'default' => 'slide',
			) );

			$element->add_control( 'bg_text_marquee', array(
				'label' => esc_html__( 'Marquee speed', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => '',
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 0,
						'max' => 10
					),
				),
				'size_units' => array( 'px' )
			) );

			$element->add_control( 'bg_text_reverse', array(
				'label' => __( 'Reverse movement', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_block' => false,
				'label_off' => __( 'Off', 'trx-effects' ),
				'label_on' => __( 'On', 'trx-effects' ),
				'default' => '',
			) );

			$element->add_control( 'bg_text_overlay', array(
				'type' => \Elementor\Controls_Manager::MEDIA,
				'label' => esc_html__( "Overlay image", 'trx-effects' ),
				'default' => array(
					'url' => '',
				),
				'selectors' => array(
					'{{WRAPPER}} .trx_effects_bg_text_overlay' => 'background-image: url({{URL}});',
				),
			) );

			$element->add_control( 'bg_text_overlay_position', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Overlay position', 'trx-effects' ),
				'label_block' => false,
				'options' => apply_filters( 'trx_effects_filter/bg_text_position', array(
					'' => esc_html__( 'Default', 'trx-effects' ),
					'top left' => esc_html__( 'Top Left', 'trx-effects' ),
					'top center' => esc_html__( 'Top Center', 'trx-effects' ),
					'top right' => esc_html__( 'Top Right', 'trx-effects' ),
					'center left' => esc_html__( 'Center Left', 'trx-effects' ),
					'center center' => esc_html__( 'Center Center', 'trx-effects' ),
					'center right' => esc_html__( 'Center Right', 'trx-effects' ),
					'bottom left' => esc_html__( 'Bottom Left', 'trx-effects' ),
					'bottom center' => esc_html__( 'Bottom Center', 'trx-effects' ),
					'bottom right' => esc_html__( 'Bottom Right', 'trx-effects' ),
				) ),
				'default' => '',
				'selectors' => array(
					'{{WRAPPER}} .trx_effects_bg_text_overlay' => 'background-position: {{VALUE}};',
				),
				'condition' => array(
					'bg_text_overlay[url]!' => ''
				),
			) );

			$element->end_controls_section();
		}
	}


	/**
	 * Add "data-trx-effects-bg-text"  to the wrapper of the element
	 *
	 * Hook render elements and add data parameters to its layout
	 *
	 * @hook action 'elementor/frontend/element/before_render'
	 * @hook action 'elementor/frontend/widget/before_render'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_data_attributes_to_section( $element ) {
		if ( is_object( $element ) && in_array( $element->get_name(), array( 'section' ) ) ) {
			$settings = $element->get_settings();
			if ( ! empty( $settings['bg_text'] ) ) {
				$element->add_render_attribute( '_wrapper', 'class', 'trx_effects_has_bg_text' );
				$element->add_render_attribute( '_wrapper', 'data-trx-effects-bg-text', json_encode( array(
					'bg_text'         => $settings['bg_text'],
					'bg_text_effect'  => $settings['bg_text_effect'],
					'bg_text_marquee' => $settings['bg_text_marquee'],
					'bg_text_reverse' => ! empty( $settings['bg_text_reverse'] ) ? 1 : 0,
					'bg_text_overlay' => $settings['bg_text_overlay'],
					'bg_text_left'    => $settings['bg_text_left'],
					'bg_text_top'     => $settings['bg_text_top'],
					)
				) );
			}
		}
	}

}
