<?php
namespace TrxEffects\Core\Modules\MouseHelper;

use TrxEffects\Plugin;
use TrxEffects\Core\Modules\Module;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class MouseHelperElementor extends Module {

	/**
	 * Class constructor.
	 *
	 * Initializing module.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}


	/**
	 * Init module
	 *
	 * Do init actions of the module
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		// Add parameter 'mouse_helper_highlight' to the 'Heading' params
		add_action( 'elementor/element/before_section_end', array( $this, 'add_highlight_params_to_heading' ), 10, 3 );

		// Add data-attributes and CSS to the 'Heading' layout
		add_action( 'elementor/frontend/element/before_render', array( $this, 'add_highlight_data_to_heading' ), 10, 1 );
		add_action( 'elementor/frontend/widget/before_render', array( $this, 'add_highlight_data_to_heading' ), 10, 1 );

		// Add module-specific parameters to the params for all Elementor's modules
		add_action( 'elementor/element/before_section_start', array( $this, 'add_params_to_all_elements' ), 10, 3 );
		add_action( 'elementor/widget/before_section_start', array( $this, 'add_params_to_all_elements' ), 10, 3 );

		// Add module-specific data-attributes to all Elementor's layouts
		add_action( 'elementor/frontend/element/before_render',  array( $this, 'add_data_to_all_elements' ), 10, 1 );
		add_action( 'elementor/frontend/section/before_render', array( $this, 'add_data_to_all_elements' ), 10, 1 );
		add_action( 'elementor/frontend/column/before_render', array( $this, 'add_data_to_all_elements' ), 10, 1 );
		add_action( 'elementor/frontend/widget/before_render', array( $this, 'add_data_to_all_elements' ), 10, 1 );
	}


	/**
	 * Highlight on mouse hover for Heading
	 *
	 * Add 'mouse_helper_highlight' to the 'Heading' params
	 *
	 * @hook action 'elementor/element/before_section_end'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_highlight_params_to_heading( $element, $section_id, $args ) {
		if ( ! is_object( $element ) ) return;
		$el_name = $element->get_name();
		if ( 'heading' == $el_name && 'section_title' === $section_id && (int) $this->get_option( 'mouse_helper' ) > 0 ) {
			$element->add_control( 'mouse_helper_highlight', array(
									'type' => \Elementor\Controls_Manager::SWITCHER,
									'label' => esc_html__( 'Highlight on mouse hover', 'trx-effects' ),
									'label_on' => esc_html__( 'On', 'trx-effects' ),
									'label_off' => esc_html__( 'Off', 'trx-effects' ),
									'return_value' => '1',
									'default' => '',
								) );
		}
	}


	/**
	 * Add data parameters and color styles to the Heading
	 *
	 * Hook render a heading and add data parameters and CSS styles to support highligh
	 *
	 * @hook action 'elementor/frontend/element/before_render'
	 * @hook action 'elementor/frontend/widget/before_render'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_highlight_data_to_heading( $element ) {
		if ( is_object( $element ) && (int) $this->get_option( 'mouse_helper' ) > 0 ) {
			$el_name = $element->get_name();
			if ( 'heading' == $el_name ) {
				$settings = $element->get_settings();
				if ( ! empty( $settings['mouse_helper_highlight'] ) ) {
					$element->add_render_attribute( 'title', 'data-trx-effects-mouse-helper', 'highlight' );
					if ( ! empty( $settings['title_color'] ) && substr( $settings['title_color'], 0, 1 ) == '#' ) {
						$element->add_render_attribute( 'title', 'class', Plugin::instance()->html->add_inline_css_class(
							'color: ' . Plugin::instance()->html->hex2rgba( $settings['title_color'], apply_filters( 'trx_effects_filter/mouse_helper_highlight_opacity', 0.33 ) ) . ' !important;'
							. 'background-image: radial-gradient(closest-side, ' . $settings['title_color'] . ' 78%, transparent 0);'
						) );
					}
				}
			}
		}
	}


	/**
	 * Mouse Helper parameters for all elements
	 *
	 * Add 'mouse_helper' parameters to the params for all Elementor's modules
	 *
	 * @hook action 'elementor/element/before_section_start'
	 * @hook action 'elementor/widget/before_section_start'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_params_to_all_elements( $element, $section_id, $args ) {

		if ( ! is_object( $element ) ) return;

		if ( in_array( $element->get_name(), array( 'section', 'column', 'common' ) ) && $section_id == '_section_responsive' && (int) $this->get_option( 'mouse_helper' ) > 0 ) {
			
			$element->start_controls_section( 'section_trx_mouse_helper', array(
																		'tab' => ! empty( $args['tab'] ) ? $args['tab'] : \Elementor\Controls_Manager::TAB_ADVANCED,
																		'label' => esc_html__( 'Mouse Helper', 'trx-effects' )
																	) );
			$element->add_control( 'mouse_helper', array(
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label' => esc_html__( 'Enable mouse helper', 'trx-effects'),
				'label_on' => esc_html__( 'On', 'trx-effects' ),
				'label_off' => esc_html__( 'Off', 'trx-effects' ),
				'return_value' => '1',
				'default' => '',
			) );

			$element->add_control( 'mouse_helper_action', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Action', 'trx-effects' ),
				'label_block' => false,
				'options' => apply_filters( 'trx_effects_filter/mouse_helper_actions', array(
					'hover' => esc_html__( 'Hover', 'trx-effects' ),
				) ),
				'condition' => array(
					'mouse_helper' => '1'
				),
				'default' => 'hover',
			) );

			if ( $this->get_option( 'mouse_helper_replace_cursor' ) != 'hide' ) {
				$element->add_control( 'mouse_helper_hide_cursor', array(
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'label' => esc_html__( 'Hide system cursor', 'trx-effects' ),
					'label_on' => esc_html__( 'On', 'trx-effects' ),
					'label_off' => esc_html__( 'Off', 'trx-effects' ),
					'return_value' => '1',
					'default' => '',
					'condition' => array(
						'mouse_helper' => '1',
					),
				) );
			}

			if ( (int) $this->get_option( 'mouse_helper_centered' ) == 0 ) {
				$element->add_control( 'mouse_helper_centered', array(
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'label' => esc_html__( 'Cursor in the center', 'trx-effects' ),
					'label_on' => esc_html__( 'On', 'trx-effects' ),
					'label_off' => esc_html__( 'Off', 'trx-effects' ),
					'return_value' => '1',
					'default' => '',
					'condition' => array(
						'mouse_helper' => '1',
					),
				) );
			}

			$element->add_control( 'mouse_helper_magnet', array(
				'label' => esc_html__( 'Magnet distance', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => 0,
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 0,
						'max' => 100
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'mouse_helper' => '1'
				),
			) );

			$element->add_control( 'mouse_helper_bg_color', array(
				'label' => esc_html__( 'Background color', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::COLOR,
				'global' => array(
					'active' => false,
				),
				'default' => '',
				'condition' => array(
					'mouse_helper' => '1',
				),
			) );

			$element->add_control( 'mouse_helper_bd_color', array(
				'label' => esc_html__( 'Border color', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::COLOR,
				'global' => array(
					'active' => false,
				),
				'default' => '',
				'condition' => array(
					'mouse_helper' => '1',
				),
			) );

			$element->add_control( 'mouse_helper_color', array(
				'label' => esc_html__( 'Text color', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::COLOR,
				'global' => array(
					'active' => false,
				),
				'default' => '',
				'condition' => array(
					'mouse_helper' => '1',
				),
			) );

			$element->add_control( 'mouse_helper_mode', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Overlay mode', 'trx-effects' ),
				'label_block' => false,
				'options' => apply_filters( 'trx_effects_filter/mouse_helper_mode', array(
					'' => esc_html__( 'Default', 'trx-effects' ),
					'normal' => esc_html__( 'Normal', 'trx-effects' ),
					'multiply'  => esc_html__( 'Multiply', 'trx-effects' ),
					'screen'  => esc_html__( 'Screen', 'trx-effects' ),
					'overlay'  => esc_html__( 'Overlay', 'trx-effects' ),
					'darken'  => esc_html__( 'Darken', 'trx-effects' ),
					'lighten'  => esc_html__( 'Lighten', 'trx-effects' ),
					'color-dodge'  => esc_html__( 'Color Dodge', 'trx-effects' ),
					'color-burn'  => esc_html__( 'Color Burn', 'trx-effects' ),
					'hard-light'  => esc_html__( 'Hard Light', 'trx-effects' ),
					'soft-light'  => esc_html__( 'Soft Light', 'trx-effects' ),
					'difference'  => esc_html__( 'Difference', 'trx-effects' ),
					'exclusion'  => esc_html__( 'Exclusion', 'trx-effects' ),
					'hue'  => esc_html__( 'Hue', 'trx-effects' ),
					'saturation'  => esc_html__( 'Saturation', 'trx-effects' ),
					'color'  => esc_html__( 'Color', 'trx-effects' ),
					'luminosity'  => esc_html__( 'Luminosity', 'trx-effects' ),
				) ),
				'condition' => array(
					'mouse_helper' => '1'
				),
				'default' => '',
			) );

			$element->add_control( 'mouse_helper_axis', array(
				'type' => \Elementor\Controls_Manager::SELECT,
				'label' => esc_html__( 'Motion axis', 'trx-effects' ),
				'label_block' => false,
				'options' => array(
					'xy' => esc_html__( 'Both', 'trx-effects' ),
					'x'  => esc_html__( 'X only', 'trx-effects' ),
					'y'  => esc_html__( 'Y only', 'trx-effects' ),
				),
				'condition' => array(
					'mouse_helper' => '1'
				),
				'default' => 'xy',
			) );

			$element->add_control( 'mouse_helper_delay', array(
				'label' => esc_html__( 'Motion delay', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => $this->get_option( 'mouse_helper_delay' ),
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 1,
						'max' => 20
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'mouse_helper' => '1'
				),
			) );

			$element->add_control( 'mouse_helper_text', array(
				'type' => \Elementor\Controls_Manager::TEXT,
				'label' => esc_html__( 'Helper text', 'trx-effects' ),
				'label_block' => false,
				'condition' => array(
					'mouse_helper' => '1'
				),
			) );

			$element->add_control( 'mouse_helper_text_size', array(
				'label' => esc_html__( 'Text size', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => '',
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 0.2,
						'max' => 2,
						'step' => 0.1
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'mouse_helper' => '1',
					'mouse_helper_text!' => ''
				),
			) );

			$element->add_control( 'mouse_helper_text_round', array(
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label' => esc_html__("Rotate text", 'trx-effects'),
				'label_on' => esc_html__( 'On', 'trx-effects' ),
				'label_off' => esc_html__( 'Off', 'trx-effects' ),
				'return_value' => '1',
				'default' => '',
				'condition' => array(
					'mouse_helper' => '1',
					'mouse_helper_text!' => ''
				),
			) );

			$element->add_control( 'mouse_helper_icon', array(
				'type' => \Elementor\Controls_Manager::ICONS,
				'label' => esc_html__( 'Icon', 'trx-effects' ),
				'label_block' => false,
				'fa4compatibility' => 'icon',
				'default' => array(
					'value' => '',
					'library' => ''
				),
				'exclude_inline_options' => array( 'svg' ),
				'skin' => 'inline',
				'condition' => array(
					'mouse_helper' => '1',
				),
			) );

			$element->add_control( 'mouse_helper_icon_size', array(
				'label' => esc_html__( 'Icon size', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => '',
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 1,
						'max' => 5,
						'step' => 0.1
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'mouse_helper' => '1',
					'mouse_helper_icon[value]!' => '',
					'mouse_helper_icon[library]!' => ''
				),
			) );

			$element->add_control( 'mouse_helper_icon_color', array(
				'label' => esc_html__( 'Icon color', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::COLOR,
				'global' => array(
					'active' => false,
				),
				'default' => '',
				'condition' => array(
					'mouse_helper' => '1',
					'mouse_helper_icon!' => array( '', 'none' ),
				),
			) );

			$element->add_control( 'mouse_helper_image', array(
				'type' => \Elementor\Controls_Manager::MEDIA,
				'label' => esc_html__( 'Image', 'trx-effects' ),
				'default' => array(
					'url' => '',
				),
				'condition' => array(
					'mouse_helper' => '1'
				),
			) );

			$element->add_control( 'mouse_helper_layout', array(
				'type' => \Elementor\Controls_Manager::TEXTAREA,
				'label' => esc_html__( 'Custom layout', 'trx-effects' ),
				'condition' => array(
					'mouse_helper' => '1'
				),
			) );

			$element->add_control( 'mouse_helper_callback', array(
				'type' => \Elementor\Controls_Manager::TEXT,
				'label' => esc_html__( 'JS Callback', 'trx-effects' ),
				'condition' => array(
					'mouse_helper' => '1'
				),
			) );

			$element->end_controls_section();
		}
	}


	/**
	 * Add "data-trx-effects-mouse-helper"  to the wrapper of the element
	 *
	 * Hook render elements and add data parameters to its layout
	 *
	 * @hook action 'elementor/frontend/element/before_render'
	 * @hook action 'elementor/frontend/section/before_render'
	 * @hook action 'elementor/frontend/column/before_render'
	 * @hook action 'elementor/frontend/widget/before_render'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_data_to_all_elements( $element ) {
		if ( is_object( $element ) ) {
			$settings = $element->get_settings();
			if ( ! empty( $settings['mouse_helper'] ) ) {
				$element->add_render_attribute( '_wrapper', array(
					'data-trx-effects-mouse-helper'            => ! empty( $settings['mouse_helper_action'] ) ? $settings['mouse_helper_action'] : 'hover',
					'data-trx-effects-mouse-helper-centered'   => (int) $this->get_option('mouse_helper_centered') == 0
														? ( ! empty( $settings['mouse_helper_centered'] ) ? $settings['mouse_helper_centered'] : 0 )
														: 1,
					'data-trx-effects-mouse-helper-magnet'     => ! empty( $settings['mouse_helper_magnet'] ) ? max( 0, $settings['mouse_helper_magnet']['size'] ) : 0,
					'data-trx-effects-mouse-helper-color'      => ! empty( $settings['mouse_helper_color'] ) ? $settings['mouse_helper_color'] : $this->get_option( 'mouse_helper_color' ),
					'data-trx-effects-mouse-helper-bg-color'   => ! empty( $settings['mouse_helper_bg_color'] ) ? $settings['mouse_helper_bg_color'] : $this->get_option( 'mouse_helper_bg_color' ),
					'data-trx-effects-mouse-helper-bd-color'   => ! empty( $settings['mouse_helper_bd_color'] ) ? $settings['mouse_helper_bd_color'] : $this->get_option( 'mouse_helper_bd_color' ),
					'data-trx-effects-mouse-helper-mode'       => ! empty( $settings['mouse_helper_mode'] ) ? $settings['mouse_helper_mode'] : '',
					'data-trx-effects-mouse-helper-axis'       => ! empty( $settings['mouse_helper_axis'] ) ? $settings['mouse_helper_axis'] : 'xy',
					'data-trx-effects-mouse-helper-delay'      => ! empty( $settings['mouse_helper_delay'] ) ?  $settings['mouse_helper_delay']['size'] : $this->get_option( 'mouse_helper_delay' ),
					'data-trx-effects-mouse-helper-text'       => ! empty( $settings['mouse_helper_text'] ) ? $settings['mouse_helper_text'] : '',
					'data-trx-effects-mouse-helper-text-size'  => ! empty( $settings['mouse_helper_text'] ) && ! empty( $settings['mouse_helper_text_size']['size'] ) ? $settings['mouse_helper_text_size']['size'] : '',
					'data-trx-effects-mouse-helper-text-round' => ! empty( $settings['mouse_helper_text_round'] ) ? $settings['mouse_helper_text_round'] : 0,
					'data-trx-effects-mouse-helper-icon'       => ! empty( $settings['mouse_helper_icon'] ) && ( empty( $settings['mouse_helper_icon']['library'] ) || $settings['mouse_helper_icon']['library'] != 'svg' ) ? trim( is_array( $settings['mouse_helper_icon'] ) ? join( ' ', $settings['mouse_helper_icon'] ) : $settings['mouse_helper_icon'] ) : '',
					'data-trx-effects-mouse-helper-icon-size'  => ! empty( $settings['mouse_helper_icon'] ) && ( empty( $settings['mouse_helper_icon']['library'] ) || $settings['mouse_helper_icon']['library'] != 'svg' ) && ! empty( $settings['mouse_helper_icon_size']['size'] ) ? $settings['mouse_helper_icon_size']['size'] : '',
					'data-trx-effects-mouse-helper-icon-color' => ! empty( $settings['mouse_helper_icon_color'] ) ? $settings['mouse_helper_icon_color'] : '',
					'data-trx-effects-mouse-helper-image'      => ! empty( $settings['mouse_helper_image']['url'] ) ? $settings['mouse_helper_image']['url'] : '',
					'data-trx-effects-mouse-helper-layout'     => ! empty( $settings['mouse_helper_layout'] ) ? $settings['mouse_helper_layout'] : '',
					'data-trx-effects-mouse-helper-callback'   => ! empty( $settings['mouse_helper_callback'] ) ? $settings['mouse_helper_callback'] : '',
				) );
				if ( ! Plugin::instance()->elementor->is_preview() ) {
					$element->add_render_attribute( '_wrapper', array(
						'data-trx-effects-mouse-helper-hide-cursor' => ! empty( $settings['mouse_helper_hide_cursor'] ) ? $settings['mouse_helper_hide_cursor'] : 0,
					) );
				}
			}
		}
	}

}
