<?php
namespace TrxEffects\Core\Options;

use TrxEffects\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Options customizer.
 *
 * Create an options page to customizer options values
 *
 * @since 1.0.0
 */
class Customizer {

	/**
	 * Options key.
	 *
	 * Name of the WordPress option with all plugin's options
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var options_key
	 */
	private $options_key = '';

	/**
	 * Options object.
	 *
	 * Holds the options object
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var options
	 */
	private $options = null;

	/**
	 * Last post type.
	 *
	 * Post type from the last field with type equal to 'post_type'.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var last_post_type
	 */

	private $last_post_type = '';


	/**
	 * Class constructor.
	 *
	 * Initializing options customizer.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct( $options ) {
		$this->options = $options;
		$this->options_key = $options->get_options_key();
		add_action( 'trx_effects_action/init', array( $this, 'save_options' ), 8 );
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}

	/**
	 * Init customizer
	 *
	 * Do init actions of the customizer
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'load_scripts' ) );
		add_action( 'admin_footer', array( $this, 'localize_scripts_admin' ) );
		add_action( 'wp_ajax_trx_effects_export_options', array( $this, 'export_options' ) );
		add_action( 'wp_ajax_trx_effects_import_options', array( $this, 'import_options' ) );
	}


	/**
	 * Add admin menu entry.
	 *
	 * Add link to the options page to the Appearance menu.
	 *
	 * @hook 'admin_menu'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function admin_menu() {
		add_theme_page(
			trx_effects_get_plugin_name(),		//page_title
			trx_effects_get_plugin_name(),		//menu_title
			'manage_options',						//capability
			"{$this->options_key}_page",			//menu_slug
			array( $this, 'options_page_builder' )	//callback
		);
	}


	/**
	 * Enqueue an admin mode scripts and styles.
	 *
	 * Load scripts and styles needs for the options page.
	 *
	 * @hook 'admin_enqueue_scripts'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load_scripts() {
		if ( apply_filters( 'trx_effects_filter/need_options', isset( $_REQUEST['page'] ) && $_REQUEST['page'] == "{$this->options_key}_page" ) ) {
			// WP styles & scripts
			wp_enqueue_style(  'wp-color-picker',      false, array(), null );
			wp_enqueue_script( 'wp-color-picker',      false, array( 'jquery' ), null, true );
			wp_enqueue_script( 'jquery-ui-tabs',       false, array( 'jquery', 'jquery-ui-core' ), null, true );
			wp_enqueue_script( 'jquery-ui-accordion',  false, array( 'jquery', 'jquery-ui-core' ), null, true );
			wp_enqueue_script( 'jquery-ui-sortable',   false, array( 'jquery', 'jquery-ui-core' ), null, true );
			wp_enqueue_script( 'jquery-ui-datepicker', false, array( 'jquery', 'jquery-ui-core' ), null, true );
			wp_enqueue_script( 'jquery-ui-slider',     false, array( 'jquery', 'jquery-ui-core' ), null, true );
			// Font with icons must be loaded before main stylesheet
			wp_enqueue_style(  'trx-effects-icons', Plugin::instance()->files->get_file_url( 'assets/css/font-icons/css/trx-effects-icons.css' ), array(), null );
			// jQuery UI skin 'Fresh'
			wp_enqueue_style(  'jquery-ui-fresh', Plugin::instance()->files->get_file_url( 'assets/css/jquery-ui-fresh.min.css' ), array(), null );
			// Internal styles & scripts
			wp_enqueue_style(  'trx-effects-icons',   Plugin::instance()->files->get_file_url( 'assets/css/font-icons/css/trx-effects-icons.css' ), array(), null );
			wp_enqueue_style(  'trx-effects-options', Plugin::instance()->files->get_file_url( 'assets/css/trx-effects-options.css' ), array(), null );
			wp_enqueue_script( 'trx-effects-utils',   Plugin::instance()->files->get_file_url( 'assets/js/trx-effects-utils.js' ), array( 'jquery' ), null, true );
			wp_enqueue_script( 'trx-effects-options', Plugin::instance()->files->get_file_url( 'assets/js/trx-effects-options.js' ), array( 'jquery' ), null, true );
			// Localize scripts
			wp_localize_script( 'trx-effects-options', 'TRX_EFFECTS_DEPENDENCIES', $this->get_dependencies() );
		}
	}


	/**
	 * Add variables to the admin mode.
	 *
	 * Add JS-vars for the options page.
	 *
	 * @hook 'admin_footer'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function localize_scripts_admin() {
		// Add variables into JS
		wp_localize_script( 'trx-effects-options', 'TRX_EFFECTS_STORAGE', apply_filters( 'trx_effects_filter/localize_script_admin', array(
			// AJAX parameters
			'ajax_url'	 => esc_url( admin_url( 'admin-ajax.php' ) ),
			'ajax_nonce' => esc_attr( wp_create_nonce( admin_url( 'admin-ajax.php' ) ) ),
			// Site base url
			'site_url'	 => esc_url( get_site_url() ),
			// Messages
			'msg_ajax_error' => esc_html__( 'Invalid server response! Try again later.', 'trx-effects' ),
			) )
		);
	}


	/**
	 * Options dependencies.
	 *
	 * Return array with options dependencies
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function get_dependencies() {
		$depends = array();
		foreach ( $this->options->get_options() as $k => $v ) {
			if ( isset( $v['dependency'] ) ) {
				$depends[ $k ] = $v['dependency'];
			}
		}
		return $depends;
	}


	/**
	 * Display options page.
	 *
	 * Build page with plugin's options.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function options_page_builder() {
		$result = Plugin::instance()->messages->get_admin_message();
		?>
		<div class="trx_effects_options">
			<h2 class="trx_effects_options_title"><?php echo esc_html( trx_effects_get_plugin_name() ); ?></h2>
			<div class="trx_effects_options_result">
				<?php
				if ( ! empty( $result['error'] ) ) {
					?><div class="error"><p><?php echo wp_kses_data( $result['error'] ); ?></p></div><?php
				} else if ( ! empty( $result['success'] ) ) {
					?><div class="updated"><p><?php echo wp_kses_data( $result['success'] ); ?></p></div><?php
				}
				?>
			</div>
			<form id="trx_effects_options_form" action="#" method="post" enctype="multipart/form-data">
				<input type="hidden" name="trx_effects_nonce" value="<?php echo esc_attr( wp_create_nonce( admin_url() ) ); ?>" />
				<?php $this->show_fields(); ?>
				<div class="trx_effects_options_buttons">
					<input type="button" class="trx_effects_options_button_submit" value="<?php esc_html_e( 'Save Options', 'trx-effects' ); ?>">
				</div>
			</form>
		</div>
		<?php		
	}


	/**
	 * Display all options fields.
	 *
	 * Display all fields from array $options separate with sections and groups.
	 *
	 * @param (boolean|array) options - an optional parameter with an array of the options to place to the form
	 * @param (boolean|string) post_type - an optional parameter with post type
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function show_fields( $options = false, $post_type = false ) {
		if ( empty( $options ) ) $options = $this->options->get_options();
		$options = apply_filters( 'trx_effects_filter/before_show_options', $options, $post_type );
		$tabs_titles = $tabs_content = $tabs_empty = array();
		$last_section = 'default';
		$last_panel = '';
		foreach ( $options as $k => $v ) {
			if ( $v['type'] == 'section' ) {
				if ( ! isset( $tabs_titles[ $k ] ) ) {
					$tabs_titles[ $k ] = $v['title'];
					$tabs_content[ $k ] = '';
					$tabs_empty[ $k ] = true;
				}
				if ( ! empty( $last_panel ) ) {
					$tabs_content[ $last_section ] .= '</div></div>';
					$last_panel = '';
				}
				$last_section = $k;
			} else if ( $v['type'] == 'panel' ) {
				if ( empty( $last_panel ) ) {
					$tabs_content[ $last_section ] = ( ! isset( $tabs_content[ $last_section ] ) ? '' : $tabs_content[ $last_section ] ) 
													. '<div class="trx_effects_panels">';
				} else {
					$tabs_content[ $last_section ] .= '</div>';
				}
				$tabs_content[ $last_section ] .= '<h4 class="trx_effects_panel_title">' . esc_html( $v['title'] ) . '</h4>'
												. '<div class="trx_effects_panel_content">';
				$last_panel = $k;
			} else if ( $v['type'] == 'panel_end' ) {
				if ( ! empty( $last_panel ) ) {
					$tabs_content[ $last_section ] .= '</div></div>';
					$last_panel = '';
				}
			} else if ( $v['type'] == 'group' ) {
				$tabs_empty[ $last_section ] = false;
				if ( count( $v['fields'] ) > 0 ) {
					$tabs_content[ $last_section ] = ( ! isset( $tabs_content[ $last_section ] ) ? '' : $tabs_content[ $last_section ] ) 
													. '<div class="trx_effects_options_group"'
														. ( isset( $v['dependency'] ) ? ' data-param="' . esc_attr( $k ) . '"' : '' )
													. '>'
														. '<h4 class="trx_effects_options_group_title'
															. ( ! empty( $v['title_class'] ) ? ' ' . esc_attr( $v['title_class'] ) : '' )
															. '">' . esc_html( $v['title'] ) . '</h4>'
														. ( ! empty( $v['override']['desc'] ) || ! empty( $v['desc'] )
															? ( '<div class="trx_effects_options_group_description">'
																. ( ! empty( $v['override']['desc'] ) 	// param 'desc' already processed with wp_kses()!
																	? trim( $v['override']['desc'] ) 
																	: ( ! empty( $v['desc'] ) ? trim( $v['desc'] ) : '' )
																	)
																. '</div><!-- /.trx_effects_options_group_description -->'
																)
															: ''
															);
					if ( ! isset( $v['val'] ) || ! is_array( $v['val'] ) || count( $v['val'] ) == 0 ) {
						$v['val'] = array( array() );
					}
					foreach ( $v['val'] as $idx => $values ) {
						$tabs_content[ $last_section ] .= '<div class="trx_effects_options_fields_set' 
															. ( ! empty( $v['clone'] ) ? ' trx_effects_options_clone' : '')
															. '">'
							. ( ! empty( $v['clone'] ) 
									? '<span class="trx_effects_options_clone_control trx_effects_options_clone_control_move"'
											. ' title="' . esc_attr__( 'Drag to sort cloned fields', 'trx-effects' ) . '"'
											. '></span>'
										. '<span class="trx_effects_options_clone_control trx_effects_options_clone_control_add"'
											. ' title="' . esc_attr__( 'Click to clone this set of the fields', 'trx-effects' ) . '"'
											. '></span>'
										. '<span class="trx_effects_options_clone_control trx_effects_options_clone_control_delete"'
											. ' title="' . esc_attr__( 'Click to delete this set of the fields', 'trx-effects' ) . '"'
											. '></span>'
									: ''
								);
						foreach ( $v['fields'] as $k1 => $v1 ) {
							$v1['val'] = isset( $values[ $k1 ] ) ? $values[ $k1 ] : $v1['std'];
							$tabs_content[ $last_section ] .= $this->show_field( $k1, $v1, "{$k}[{$idx}]" );
						}
						$tabs_content[ $last_section ] .= '</div>';
					}
					$tabs_content[ $last_section ] .= '</div>';
				}
			} else {
				if ( empty( $v['hidden'] ) && $v['type'] != 'hidden' ) {
					$tabs_empty[ $last_section ] = false;
				}
				$tabs_content[ $last_section ] = ( ! isset( $tabs_content[ $last_section ] ) ? '' : $tabs_content[ $last_section ] ) 
												. $this->show_field( $k, $v );
			}
		}
		if ( ! empty( $last_panel ) ) {
			$tabs_content[ $last_section ] .= '</div></div>';
		}
		
		if ( count( $tabs_content ) > 0 ) {
			?>
			<div class="trx_effects_tabs <?php echo count( $tabs_titles ) > 1 ? 'with_tabs' : 'no_tabs'; ?>">
				<?php
				if ( count( $tabs_titles ) > 1 ) {
					?><ul><?php
						foreach ( $tabs_titles as $k => $v ) {
							if ( empty( $tabs_content[ $k ] ) || $tabs_empty[ $k ] ) continue;
							?><li<?php if ( $tabs_empty[ $k ] ) echo ' class="trx_effects_options_item_hidden"'; ?>><?php
								?><a href="#trx_effects_tabs_section_<?php echo esc_attr( $k ); ?>"><?php echo esc_html( $v ); ?></a><?php
							?></li><?php
						}
					?></ul>
				<?php
				}
				foreach ( $tabs_content as $k => $v ) {
					if ( empty( $v ) || $tabs_empty[ $k ] ) continue;
					?>
					<div id="trx_effects_tabs_section_<?php echo esc_attr( $k ); ?>" class="trx_effects_tabs_section trx_effects_options_section">
						<?php Plugin::instance()->html->show_layout( $v ); ?>
					</div>
					<?php
				}
				?>
			</div>
			<?php
		}
	}


	/**
	 * Display a single field.
	 *
	 * Display a single field by its type.
	 *
	 * @param (string) name - a name of the field
	 * @param (array) field - an array with field description
	 * @param (string) group - an optional parameter with a name of the group
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function show_field( $name, $field, $group = '' ) {
		$class_name = 'TrxEffects\Core\Options\CustomizerField' . ucfirst( $field['type'] );
		$field_obj  = new $class_name( $this );
		return $field_obj->show_field( $name, $field, $group );
	}


	/**
	 * Get complex field value from the POST
	 *
	 * Convert string to the complex value (array) before save options
	 *
	 * @param (string) name - a name of the query parameter
	 * @param (array) field - an array with field options
	 *
	 * @return (number|string|array) val - a value catched from a query string
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function get_field_value( $name, $field ) {
		$key = "{$this->options_key}_field_{$name}";
		$val = isset( $_POST[ $key ] )
							? Plugin::instance()->html->get_value_gp( $key )
							: ( $field['type'] == 'checkbox' ? 0 : '' );
		if ( is_array( $field['std'] ) && ! is_array( $val ) ) {
			if ( ! empty( $val ) ) {
				$tmp = explode( '|', $val );
				$val = array();
				foreach ( $tmp as $v ) {
					$v = explode( '=', $v );
					$val[ $v[0] ] = $v[1];
				}
			} else {
				$val = array();
			}
		}
		return $val;
	}


	/**
	 * Save options.
	 *
	 * Save options as an array to the WordPress Options table.
	 *
	 * @hook 'trx_effects_action/init' priority 8 (after options loaded on priority 4)
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function save_options() {

		if ( ! isset( $_REQUEST['page'] ) || $_REQUEST['page'] != "{$this->options_key}_page" || Plugin::instance()->html->get_value_gp( 'trx_effects_nonce' ) == '' ) return;

		// Verify nonce
		if ( ! wp_verify_nonce( Plugin::instance()->html->get_value_gp( 'trx_effects_nonce' ), admin_url() ) ) {
			Plugin::instance()->messages->set_admin_message( esc_html__( 'Bad security code! Options are not saved!', 'trx-effects'), 'error' );
			return;
		}

		// Check permissions
		if ( ! current_user_can( 'manage_options' ) ) {
			Plugin::instance()->messages->set_admin_message( esc_html__('Manage options is denied for the current user! Options are not saved!', 'trx-effects'), 'error' );
			return;
		}

		// Save options
		$values = array();
		foreach ( $this->options->get_options() as $k => $v ) {
			// Skip options without value (section, info, etc.)
			if ( ! isset( $v['std'] ) ) continue;
			// Get option value from POST
			if ( ! empty( $v['hidden'] ) ) {
				if ( $v['std'] != $v['val'] ) {
					$values[$k] = $v['val'];
				}
			} else {
				$values[ $k ] = $this->get_field_value( $k, $v );
			}
		}
		update_option( $this->options_key, apply_filters( 'trx_effects_filter/save_options', $values ) );

		// Set message for next session
		Plugin::instance()->messages->set_admin_message( esc_html__('Options are saved', 'trx-effects'), 'success', true );

		if ( ! empty( $_SERVER['HTTP_REFERER'] ) ) {
			wp_safe_redirect( $_SERVER['HTTP_REFERER'] );
			exit();
		}
	}


	/**
	 * Export options.
	 *
	 * Export options as a serialized string.
	 *
	 * @hook 'wp_ajax_trx_effects_export_options'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function export_options() {

		$response = array(
			'error' => '',
			'success' => ''
		);

		// Verify nonce
		if ( ! wp_verify_nonce( Plugin::instance()->html->get_value_gp( 'nonce' ), admin_url( 'admin-ajax.php' ) ) ) {
			$response['error'] = esc_html__( 'Illegal secret code', 'trx-effects' );

		// Check permissions
		} else if ( ! current_user_can( 'manage_options' ) ) {
			$response['error'] = esc_html__( 'You have not permissions to manage options on this site', 'trx-effects' );

		// Export options
		} else {
			$response['success'] = serialize( apply_filters('trx_effects_filter/load_options', get_option( $this->options_key, array() ) ) );
		}

		echo json_encode( $response );
		die();
	}


	/**
	 * Import options.
	 *
	 * Import options as a serialized string.
	 *
	 * @hook 'wp_ajax_trx_effects_import_options'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function import_options() {

		$response = array(
			'error' => '',
			'success' => ''
		);

		// Verify nonce
		if ( ! wp_verify_nonce( Plugin::instance()->html->get_value_gp( 'nonce' ), admin_url( 'admin-ajax.php' ) ) ) {
			$response['error'] = esc_html__( 'Illegal secret code', 'trx-effects' );

		// Check permissions
		} else if ( ! current_user_can( 'manage_options' ) ) {
			$response['error'] = esc_html__( 'You have not permissions to manage options on this site', 'trx-effects' );

		// Verify options
		} else if ( ( $options = Plugin::instance()->html->get_value_gp( 'options' ) ) == '' || ! is_serialized( $options ) ) {
			$response['error'] = esc_html__( 'Incorrect data to import', 'trx-effects' );

		// Import options
		} else {
			$options = maybe_unserialize( $options );
			if ( is_array( $options ) ) {
				update_option( $this->options_key, apply_filters( 'trx_effects_filter/save_options', $options ) );
				$response['success'] = esc_html__( 'Options are imported', 'trx-effects' );
				Plugin::instance()->messages->set_admin_message( $response['success'], 'success', true );
			}
		}

		echo json_encode( $response );
		die();
	}

}
