<?php
namespace TrxEffects\Core\Options;

use TrxEffects\Plugin;
use TrxEffects\Core\Options\Customizer;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * General Options class.
 *
 * Handling plugin's options
 *
 * @since 1.0.0
 */
class Options {

	/**
	 * Options key.
	 *
	 * Name of the WordPress option with all plugin's options
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var options_key
	 */
	private $options_key = 'trx_effects_options';

	/**
	 * Options list.
	 *
	 * Holds the array with all plugin's option values
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var options
	 */
	private $options = false;

	/**
	 * Options customizer.
	 *
	 * Holds the reference to the object 'OptionsCustomizer'
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var customizer
	 */
	private $customizer = null;


	/**
	 * Class constructor.
	 *
	 * Initializing options.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/load', array( $this, 'load' ) );
		add_action( 'trx_effects_action/init', array( $this, 'load_options' ), 4 );
	}

	/**
	 * Load.
	 *
	 * Load customizer if is admin mode.
	 *
	 * @hook action 'trx_effects_action/load'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load() {
		if ( is_admin() ) {
			$this->customizer = new Customizer( $this );
		}
	}


	/**
	 * Load options
	 *
	 * Load an array with saved options
	 *
	 * @hook 'trx_effects_action/init' priority 4
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load_options() {
		$options = $this->get_options_list();
		$values  = apply_filters( 'trx_effects_filter/load_options', get_option( $this->options_key, array() ) );
		foreach( $options as $k => $v ) {
			if ( isset( $v['std'] ) ) {
				$options[ $k ]['val'] = isset( $_GET[ $k ] )
											? wp_unslash( $_GET[ $k ] )
											: ( isset( $values[ $k ] )
												? $values[ $k ]
												: $v['std']
												);
			}
		}
		$this->options = $options;
	}


	/**
	 * Check if option name exists
	 *
	 * Return true if an option with specified name exists.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function check_option( $name ) {
		if ( $this->options === false ) {
			$this->load_options();
		}
		return isset( $this->options[ $name ] );
	}


	/**
	 * Return an option value
	 *
	 * Return value by option name.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_option( $name, $defa = '', $strict_mode = true ) {
		if ( $this->options === false ) {
			$this->load_options();
		}
		$rez = $defa;
		$part = '';
		if ( strpos( $name, '[' ) !== false ) {
			$tmp  = explode( '[', $name );
			$name = $tmp[0];
			$part = substr( $tmp[1], 0, -1 );
		}
		// If options are loaded and specified name is not exists and 'strict_mode' is on -
		// display warning message and dump call's stack
		if ( ! $this->check_option( $name ) && $strict_mode ) {
			$s = '';
			if ( function_exists( 'ddo' ) ) {
				$s = debug_backtrace();
				array_shift($s);
				$s = ddo($s, 0, 3);
			}
			wp_die(
				// Translators: Add option's name to the message
				esc_html( sprintf( __( 'Undefined option "%s"', 'trx-effects' ), $name ) )
				. ( ! empty( $s )
						? ' ' . esc_html__( 'called from:', 'trx-effects' ) . "<pre>" . wp_kses_data( $s ) . '</pre>'
						: ''
						)
			);
		}
		// Override option from GET
		if ( isset( $_GET[ $name ] ) ) {
			if ( empty( $part ) ) {
				$rez = wp_unslash( $_GET[ $name ] );
			} else if ( isset( $_GET[ $name ][ $part ] ) ) {
				$rez = wp_unslash( $_GET[ $name ][ $part ] );
			}
		// Get saved option value
		} else {
			if ( empty( $part ) ) {
				$rez = $this->options[ $name ][ 'val' ];
			} else if ( isset( $this->options[ $name ]['val'][ $part ] ) ) {
				$rez = $this->options[ $name ]['val'][ $part ];
			}
		}
		return $rez;
	}


	/**
	 * Get an options key
	 *
	 * Return value of the property 'options_key'.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_options_key() {
		return $this->options_key;
	}


	/**
	 * Get the array with options
	 *
	 * Return value of the property 'options' - array with options definitions and values
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_options() {
		return $this->options;
	}


	/**
	 * Plugin's options.
	 *
	 * Return array with plugin's options
	 *
	 * @return (array) $options
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_options_list() {
		$modules = Plugin::instance()->manager->get_list_modules();
		$keys = array();
		foreach( $modules as $k => $v ) {
			$keys[ $k ] = 1;
		}
		return apply_filters( 'trx_effects_filter/options_list', array(
			'general_section' => array(
				"title" => esc_html__('General', 'trx-effects'),
				"desc" => esc_html__('General settings', 'trx-effects' ),
				"type" => "section"
			),
			'general_info' => array(
				"title" => esc_html__('General', 'trx-effects'),
				"desc" => esc_html__( 'General settings of the plugin', 'trx-effects' ),
				"type" => "info"
			),
			'modules' => array(
				"title" => esc_html__('Modules', 'trx-effects'),
				"desc" => esc_html__( 'List of the available modules to load. Attention! If you disable a module, its settings are reset to the default values.', 'trx-effects' ),
				"options" => $modules,
				"std" => $keys,
				"type" => "checklist"
			),
			'page_wrap_class' => array(
				"title" => esc_html__('Theme-specific page wrap class', 'trx-effects'),
				"desc" => esc_html__( 'Selector of the theme-specific page wrapper', 'trx-effects' ),
				"std" => ".page_wrap",
				"type" => "text"
			),
		) );
	}

}
