<?php
namespace TrxEffects;

use TrxEffects\Core\Base;
use TrxEffects\Core\Utils\Files;
use TrxEffects\Core\Utils\Html;
use TrxEffects\Core\Utils\Messages;
use TrxEffects\Core\Api\Elementor;
use TrxEffects\Core\Options\Options;
use TrxEffects\Core\Modules\Manager as ModulesManager;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once TRX_EFFECTS_DIR . 'core/base.php';

/**
 * Main plugin class.
 *
 * The main plugin handler class is responsible for initializing plugin functionality.
 * The class registers and all the components required to run the plugin.
 *
 * @since 1.0.0
 */
class Plugin extends Base {

	/**
	 * File system.
	 *
	 * Holds the object with file system utilities.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var files
	 */
	public $files;

	/**
	 * Options object.
	 *
	 * Holds the options object
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var options
	 */
	public $options = null;

	/**
	 * Messages subsystem.
	 *
	 * Holds the object of the messages subsystem.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var messages
	 */
	public $messages;

	/**
	 * HTML utilities.
	 *
	 * Holds the object with HTML and CSS utilities.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var html
	 */
	public $html;

	/**
	 * Elementor API object.
	 *
	 * Holds the object with Elementor API.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var elementor
	 */
	public $elementor;

	/**
	 * Modules manager.
	 *
	 * Holds the object of the modules manager.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var manager
	 */
	public $manager;

	/**
	 * Instance.
	 *
	 * Holds singleton instance.
	 *
	 * @since 1.0.0
	 * @access private
	 * @static
	 *
	 * @var instances
	 */
	private static $instance = null;

	/**
	 * Instance.
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @access public
	 * @static
	 *
	 * @return Instance of the class.
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}


	/**
	 * Plugin constructor.
	 *
	 * Initializing plugin.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		// Register class autoloader
		$this->register_autoloader();

		// Load plugin's components
		$this->load();

		// Init plugin's components
		add_action( 'init', array( $this, 'init' ) );
	}

	/**
	 * Load.
	 *
	 * Load plugin's components.
	 *
	 * @hook action 'init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load() {

		$this->files = new Files();
		$this->options = new Options();
		$this->messages = new Messages();
		$this->html = new Html();
		$this->elementor = new Elementor();
		$this->manager = new ModulesManager();
	}

	/**
	 * Init.
	 *
	 * Initialize Plugin.
	 *
	 * @hook action 'init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		add_action("wp_enqueue_scripts", array( $this, 'load_scripts' ) );
		add_action("wp_footer", array( $this, 'localize_scripts_front' ) );

		/**
		 * Plugin init.
		 *
		 * Fires on plugin init, after a plugin has finished loading
		 * but before any headers are sent.
		 *
		 * @since 1.0.0
		 */
		do_action( 'trx_effects_action/init' );
	}


	/**
	 * Enqueue scripts and styles.
	 *
	 * Load an frontend mode scripts and styles.
	 *
	 * @hook 'wp_enqueue_scripts'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load_scripts() {
		// Font with icons must be loaded before main stylesheet
		wp_enqueue_style(  'trx-effects-icons', $this->files->get_file_url( 'assets/css/font-icons/css/trx-effects-icons.css' ), array(), null );
		wp_enqueue_style(  'trx-effects', $this->files->get_file_url( 'assets/css/trx-effects.css' ), array(), null );
		wp_enqueue_script( 'trx-effects-utils',   $this->files->get_file_url( 'assets/js/trx-effects-utils.js' ), array( 'jquery' ), null, true );
		wp_enqueue_script( 'trx-effects',   $this->files->get_file_url( 'assets/js/trx-effects.js' ), array( 'jquery' ), null, true );
	}

	/**
	 * Localize frontend scripts.
	 *
	 * Fires on wp_footer, after all modules output sent.
	 *
	 * @hook action 'wp_footer'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function localize_scripts_front() {
		wp_localize_script( 'trx-effects-utils', 'TRX_EFFECTS_STORAGE', apply_filters( 'trx_effects_filter/localize_scripts_front', array(
			// AJAX parameters
			'ajax_url'	=> esc_url( admin_url( 'admin-ajax.php' ) ),
			'ajax_nonce'=> esc_attr( wp_create_nonce( admin_url( 'admin-ajax.php' ) ) ),
			// User logged in
			'user_logged_in' => is_user_logged_in(),
			// Site base url
			'site_url'	=> esc_url( get_home_url() ),
			// Selector of the theme-specific page wrapper
			'page_wrap' => $this->options->get_option( 'page_wrap_class', '.page_wrap' ),
		) ) );
	}


	/**
	 * Enqueue a script 'Curtains'
	 *
	 * Load a script 'curtains.js' supports WebGL canvas drawing.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load_script_curtains() {
		wp_enqueue_script( 'curtains', $this->files->get_file_url( 'assets/js/curtains/curtains.min.js' ), array(), null, true );
	}


	/**
	 * Enqueue a script 'TweenMax'
	 *
	 * Load a script 'tweenmax.js' supports WebGL canvas drawing.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load_script_tweenmax() {
		wp_enqueue_script( 'tweenmax', $this->files->get_file_url( 'assets/js/tweenmax/tweenmax.min.js' ), array(), null, true );
	}


	/**
	 * Register autoloader.
	 *
	 * Autoloader loads all the classes needed to run the plugin.
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function register_autoloader() {
		require_once TRX_EFFECTS_DIR . '/core/autoloader.php';
		Autoloader::run();
	}
}

Plugin::instance();

/**
 * Plugin is loaded.
 *
 * Fires when plugin was fully loaded and instantiated.
 *
 * @since 1.0.0
 */
do_action( 'trx_effects_action/load' );
