<?php
namespace TrxEffects\Core\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * HTML and CSS utilities.
 *
 * Utilities for manipulation with html layouts and css rules
 *
 * @since 1.0.0
 */
class Html {

	/**
	 * Inline CSS code.
	 *
	 * Hold a CSS code to append inline to the body.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var inline_css
	 */
	private $inline_css = '';

	/**
	 * Inline HTML code.
	 *
	 * Hold a HTML code to append inline to the body.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var inline_html
	 */
	private $inline_html = '';


	/**
	 * Class constructor.
	 *
	 * Initializing class properties.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}


	/**
	 * Init module
	 *
	 * Do init actions of the module
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {
		// Put inline CSS to the page
		add_action( 'wp_footer', array( $this, 'put_inline_css' ), 9999 );
		// Put inline HTML to the page
		add_action( 'wp_footer', array( $this, 'put_inline_html' ), 9999 );
	}


	/**
	 * Output any html layout.
	 *
	 * Output string with the html layout (if not empty)
	 * (put it between 'before' and 'after' tags)
	 * Attention! This string may contain layout formed in any plugin (widgets or shortcodes output)
	 * and not require escaping to prevent damage!
	 *
	 * @param (string) str - string to output
	 * @param (string) before - (optional) string to output before
	 * @param (string) after - (optional) string to output after
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function show_layout( $str, $before = '', $after = '' ) {
		if ( trim( $str ) != '' ) {
			printf( "%s%s%s", $before, $str, $after );
		}
	}


	/**
	 * Add units to the CSS value.
	 *
	 * Return value for the style attr with units ('px' is added if units are absent)
	 *
	 * @param (string) val - value with or without units
	 *
	 * @return (string) val - value with units
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function prepare_css_value( $val ) {
		if ( $val != '' ) {
			$ed = substr( $val, -1 );
			if ( '0' <= $ed && $ed <= '9' ) $val .= 'px';
		}
		return $val;
	}



	/* URL manipulations
	-------------------------------------------------------------------------------- */

	/**
	 * Remove protocol from URL.
	 *
	 * Return URL without protocol
	 *
	 * @param (string) url - URL with or without protocol
	 * @param (boolean) complete - true if complete protocol should be removed or false if only http[s]: part must be removed
	 *                             ( '//' leave in the url )
	 *
	 * @return (string) url - URL without protocol
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function remove_protocol( $url, $complete = false ) {
		$url = preg_replace( '/http[s]?:' . ( $complete ? '\\/\\/' : '' ) . '/', '', $url );
		return $url;
	}


	/**
	 * Check if string is contain an URL.
	 *
	 * Return true if a string is contain an URL
	 *
	 * @param (string) url - string to check
	 *
	 * @return (boolean) - true if string contain an URL, otherwise - false
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function is_url( $url ) {
		return strpos( $url, '://' ) !== false;
	}


	/**
	 * Check if string is an external URL.
	 *
	 * Return true if a string is an external URL
	 *
	 * @param (string) url - string to check
	 *
	 * @return (boolean) - true if string is an external URL, otherwise - false
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function is_external_url( $url ) {
		return $this->is_url( $url ) && strpos( $url, $this->remove_protocol( get_home_url(), true ) ) === false;
	}


	/**
	 * Return current page URL.
	 *
	 * Return URL of the current page
	 *
	 * @return (string) url - URL of the current page
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_current_url() {
		return add_query_arg( array() );
	}


	/**
	 * Add parameters to the URL.
	 *
	 * Add multiple parameters to the URL to send its as GET query
	 *
	 * @param (string) url - string with url
	 * @param (array) prm - associative array with parameters
	 * 
	 * @return (string) url - new url with parameters added
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function add_to_url( $url, $prm ) {
		if ( is_array( $prm ) && count( $prm ) > 0 ) {
			$separator = strpos( $url, '?' ) === false ? '?' : '&';
			foreach ( $prm as $k => $v ) {
				$url .= $separator . urlencode( $k ) . '=' . urlencode( $v );
				$separator = '&';
			}
		}
		return $url;
	}


	/**
	 * Check if URL contain substring.
	 *
	 * Return true if URL contain any specified string
	 *
	 * @param (string|array) val - string or array of string to search
	 * 
	 * @return (boolean) rez - true if any string from val occur in the current page URL
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function check_url( $val='' ) {
		if ( ! is_array( $val ) ) $val = array( $val );
		$rez = false;
		foreach	( $val as $s ) {
			$rez = strpos( $_SERVER['REQUEST_URI'], $s ) !== false;
			if ($rez) break;
		}
		return $rez;
	}


	/**
	 * Check if current mode is post edit.
	 *
	 * Return true if a current mode is a post editor
	 *
	 * @return (boolean) mode
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function is_post_edit() {
		return ( $this->check_url( 'post.php' ) && ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' )
				||
				$this->check_url( 'post-new.php' )
				||
				( ! empty( $_GET['context'] ) && $_GET['context'] == 'edit' && ! empty( $_GET['post_id'] ) && $_GET['post_id'] > 0 );
	}



	/* Add inline CSS and HTML
	-------------------------------------------------------------------------------- */

	/**
	 * Generate value for the attribute 'id'
	 *
	 * Generate value for the attribute 'id' with specified prefix (optional)
	 *
	 * @param (string) prefix - (optional) prefix placed before random id
	 * 
	 * @return (string) id - generated id
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function generate_id( $prefix = '' ) {
		return $prefix . str_replace( '.', '', mt_rand() );
	}


	/**
	 * Get class for dynamic CSS rules
	 *
	 * Add CSS rules to the inline CSS holder and return class for it
	 *
	 * @param (string) css - CSS rules to place to the holder
	 * @param (string) suffix - (optional) suffix placed after class name
	 * 
	 * @return (string) class_name - generated class name
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function add_inline_css_class( $css, $suffix='' ) {
		$class_name = $this->generate_id( 'trx_effects_inline_' );
		$this->add_inline_css(
			sprintf(
				'.%s%s{%s}',
				$class_name,
				! empty( $suffix ) 
					? ( substr( $suffix, 0, 1 ) != ':' ? ' ' : '' ) . str_replace( ',', ",.{$class_name} ", $suffix )
					: '',
				$css
			)
		);
		return $class_name;
	}


	/**
	 * Add CSS rules to the inline CSS holder
	 *
	 * Add CSS rules to the inline CSS holder to place its to the page footer
	 *
	 * @param (string) css - CSS rules to place to the holder
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function add_inline_css( $css ) {
		$this->inline_css .= $css;
	}


	/**
	 * Return inline CSS holder
	 *
	 * Return dynamic CSS to insert it to the footer
	 *
	 * @param (boolean) clear - remove any CSS from the holder after return it
	 * 
	 * @return (string) inline_css - holder's value with inline css
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function get_inline_css( $clear = false ) {
		$rez = $this->inline_css;
		if ( $clear ) $this->inline_css = '';
		return $rez;
	}


	/**
	 * Put inline CSS to the page
	 *
	 * Place inline CSS to the page footer
	 * 
	 * @hook 'wp_footer' with priority 9999
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function put_inline_css() {
		$this->show_layout(
			apply_filters( 'trx_effects_filter/put_inline_css', $this->get_inline_css( true ) ),
			'<style type="text/css" id="trx_effects-inline-styles-inline-css">',
			'</style>'
		);
	}


	/**
	 * Add HTML layout to the inline HTML holder
	 *
	 * Add HTML layout to the inline HTML holder to place its to the page footer
	 *
	 * @param (string) html - HTML layout to place to the holder
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function add_inline_html( $html ) {
		$this->inline_html .= $html;
	}


	/**
	 * Return inline HTML holder
	 *
	 * Return dynamic HTML to insert it to the footer
	 *
	 * @param (boolean) clear - remove any HTML from the holder after return it
	 * 
	 * @return (string) inline_html - holder's value with inline HTML
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function get_inline_html( $clear = false ) {
		$rez = $this->inline_html;
		if ( $clear ) $this->inline_html = '';
		return $rez;
	}


	/**
	 * Put inline HTML to the page
	 *
	 * Place inline HTML to the page footer
	 * 
	 * @hook 'wp_footer' with priority 9999
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function put_inline_html() {
		$this->show_layout( apply_filters( 'trx_effects_filter/put_inline_html', $this->get_inline_html( true ) ) );
	}



	/* GET, POST and SESSION utilities
	-------------------------------------------------------------------------------- */

	/**
	 * Strip slashes from value
	 *
	 * Strip slashes if Magic Quotes is on
	 *
	 * @param (string) val - value from GET or POST or COOKIE or SESSION with slashes
	 * 
	 * @return (string) val - unslashed value
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function stripslashes( $val ) {
		static $magic = 0;
		if ( $magic === 0 ) {
			$magic = version_compare( phpversion(), '5.4', '>=' )
					|| ( function_exists( 'get_magic_quotes_gpc' ) && get_magic_quotes_gpc() == 1 ) 
					|| ( function_exists( 'get_magic_quotes_runtime') && get_magic_quotes_runtime() == 1 ) 
					|| strtolower( ini_get( 'magic_quotes_sybase' ) ) == 'on';
		}
		if ( is_array( $val ) ) {
			foreach( $val as $k => $v ) {
				$val[$k] = $this->stripslashes( $v );
			}
		} else {
			$val = $magic ? stripslashes( trim( $val ) ) : trim( $val );
		}
		return $val;
	}


	/**
	 * Return GET or POST value
	 *
	 * Return value from GET or POST
	 *
	 * @param (string) name - name to seek in GET or POST
	 * @param (mixed) defa - default value (if not sent)
	 * 
	 * @return (string) val - unslashed value
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function get_value_gp( $name, $defa = '' ) {
		if ( isset( $_GET[ $name ] ) )			$rez = $_GET[ $name ];
		else if ( isset( $_POST[ $name ] ) )	$rez = $_POST[ $name ];
		else									$rez = $defa;
		return $this->stripslashes( $rez );
	}


	/**
	 * Return GET or POST or COOKIE value
	 *
	 * Return value from GET or POST or COOKIE
	 *
	 * @param (string) name - name to seek in GET or POST or COOKIE
	 * @param (mixed) defa - default value (if not sent)
	 * 
	 * @return (string) val - unslashed value
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function get_value_gpc( $name, $defa = '' ) {
		if ( isset( $_GET[ $name ] ) )			$rez = $_GET[ $name ];
		else if ( isset( $_POST[ $name ] ) )	$rez = $_POST[ $name ];
		else if ( isset( $_COOKIE[ $name ] ) )	$rez = $_COOKIE[ $name ];
		else									$rez = $defa;
		return $this->stripslashes( $rez );
	}


	/**
	 * Return GET or POST or SESSION value
	 *
	 * Return value from GET or POST or SESSION
	 *
	 * @param (string) name - name to seek in GET or POST or SESSION
	 * @param (mixed) defa - default value (if not sent)
	 * 
	 * @return (string) val - unslashed value
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function get_value_gps( $name, $defa = '' ) {
		if ( isset( $_GET[ $name ] ) )			$rez = $_GET[$name];
		else if ( isset( $_POST[ $name ] ) )	$rez = $_POST[$name];
		else if ( isset( $_SESSION[ $name ] ) )	$rez = $_SESSION[$name];
		else									$rez = $defa;
		return $this->stripslashes( $rez );
	}



	/* Colors manipulations
	-------------------------------------------------------------------------------- */

	/**
	 * Convert hex to rgb
	 *
	 * Convert color from hex value #RRGGBB to rgb array( 'r' => rr, 'g' => gg, 'b' => bb )
	 *
	 * @param (string) hex - color value as hex
	 * 
	 * @return (array) val - color value as rgb array
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function hex2rgb( $hex ) {
		$dec = hexdec( substr( $hex, 0, 1 ) == '#' ? substr( $hex, 1 ) : $hex );
		return array( 'r'=> $dec >> 16, 'g' => ( $dec & 0x00FF00 ) >> 8, 'b' => $dec & 0x0000FF );
	}


	/**
	 * Convert hex to rgba
	 *
	 * Convert color from hex value #RRGGBB to rgba(rr, gg, bb, alpha )
	 *
	 * @param (string) hex - color value as hex
	 * @param (string) alpha - alpha component
	 * 
	 * @return (array) val - color value as rgba
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function hex2rgba( $hex, $alpha ) {
		$rgb = $this->hex2rgb( $hex );
		return 'rgba(' . $rgb['r'] . ',' . $rgb['g'] . ',' . $rgb['b'] . ',' . $alpha . ')';
	}


	/**
	 * Convert hex to hsb
	 *
	 * Convert color from hex value #RRGGBB to hsb array( 'h' => hue, 's' => saturation, 'b' => brightness )
	 *
	 * @param (string) hex - color value as hex
	 * @param (int) h - shift for 'h' component
	 * @param (int) s - shift for 's' component
	 * @param (int) b - shift for 'b' component
	 * 
	 * @return (array) val - color value as hsb array
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function hex2hsb( $hex, $h=0, $s=0, $b=0 ) {
		$hsb = $this->rgb2hsb( $this->hex2rgb( $hex ) );
		$hsb['h'] = min( 359, max( 0, $hsb['h'] + $h ) );
		$hsb['s'] = min( 100, max( 0, $hsb['s'] + $s ) );
		$hsb['b'] = min( 100, max( 0, $hsb['b'] + $b ) );
		return $hsb;
	}


	/**
	 * Convert rgb to hsb
	 *
	 * Convert color from rgb value (array) to hsb array( 'h' => hue, 's' => saturation, 'b' => brightness )
	 *
	 * @param (array) rgb - color value as rgb array
	 * 
	 * @return (array) val - color value as hsb array
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function rgb2hsb( $rgb ) {
		$hsb = array();
		$hsb['b'] = max( max( $rgb['r'], $rgb['g'] ), $rgb['b'] );
		$hsb['s'] = $hsb['b'] <= 0 ? 0 : round( 100 * ( $hsb['b'] - min( min( $rgb['r'], $rgb['g'] ), $rgb['b'] ) ) / $hsb['b'] );
		$hsb['b'] = round( ( $hsb['b'] / 255 ) * 100 );
		if ( ( $rgb['r'] == $rgb['g'] ) && ( $rgb['g'] == $rgb['b'] ) ) $hsb['h'] = 0;
		else if ( $rgb['r'] >= $rgb['g'] && $rgb['g'] >= $rgb['b'] ) $hsb['h'] =  60 * ( $rgb['g'] - $rgb['b'] ) / ( $rgb['r'] - $rgb['b'] );
		else if ( $rgb['g'] >= $rgb['r'] && $rgb['r'] >= $rgb['b'] ) $hsb['h'] =  60 + 60 * ( $rgb['g'] - $rgb['r'] ) / ( $rgb['g'] - $rgb['b'] );
		else if ( $rgb['g'] >= $rgb['b'] && $rgb['b'] >= $rgb['r'] ) $hsb['h'] = 120 + 60 * ( $rgb['b'] - $rgb['r'] ) / ( $rgb['g'] - $rgb['r'] );
		else if ( $rgb['b'] >= $rgb['g'] && $rgb['g'] >= $rgb['r'] ) $hsb['h'] = 180 + 60 * ( $rgb['b'] - $rgb['g'] ) / ( $rgb['b'] - $rgb['r'] );
		else if ( $rgb['b'] >= $rgb['r'] && $rgb['r'] >= $rgb['g'] ) $hsb['h'] = 240 + 60 * ( $rgb['r'] - $rgb['g'] ) / ( $rgb['b'] - $rgb['g'] );
		else if ( $rgb['r'] >= $rgb['b'] && $rgb['b'] >= $rgb['g'] ) $hsb['h'] = 300 + 60 * ( $rgb['r'] - $rgb['b'] ) / ( $rgb['r'] - $rgb['g'] );
		else $hsb['h'] = 0;
		$hsb['h'] = round( $hsb['h'] );
		return $hsb;
	}


	/**
	 * Convert hsb to rgb
	 *
	 * Convert color from hsb value (array) to rgb array
	 *
	 * @param (array) hsb - color value as hsb array
	 * 
	 * @return (array) rgb - color value as rgb array
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function hsb2rgb( $hsb ) {
		$rgb = array();
		$h = round( $hsb['h'] );
		$s = round( $hsb['s'] * 255 / 100 );
		$v = round( $hsb['b'] * 255 / 100 );
		if ( $s == 0 ) {
			$rgb['r'] = $rgb['g'] = $rgb['b'] = $v;
		} else {
			$t1 = $v;
			$t2 = ( 255 - $s ) * $v / 255;
			$t3 = ( $t1 - $t2 ) * ( $h % 60 ) / 60;
			if ( $h == 360 ) $h = 0;
			if (      $h <  60 ) {	$rgb['r'] = $t1; $rgb['b'] = $t2; $rgb['g'] = $t2 + $t3; }
			else if ( $h < 120 ) {	$rgb['g'] = $t1; $rgb['b'] = $t2; $rgb['r'] = $t1 - $t3; }
			else if ( $h < 180 ) {	$rgb['g'] = $t1; $rgb['r'] = $t2; $rgb['b'] = $t2 + $t3; }
			else if ( $h < 240 ) {	$rgb['b'] = $t1; $rgb['r'] = $t2; $rgb['g'] = $t1 - $t3; }
			else if ( $h < 300 ) {	$rgb['b'] = $t1; $rgb['g'] = $t2; $rgb['r'] = $t2 + $t3; }
			else if ( $h < 360 ) {	$rgb['r'] = $t1; $rgb['g'] = $t2; $rgb['b'] = $t1 - $t3; }
			else {					$rgb['r'] = 0;   $rgb['g'] = 0;   $rgb['b'] = 0; }
		}
		return array( 'r' => round( $rgb['r'] ), 'g' => round( $rgb['g'] ), 'b' => round( $rgb['b'] ) );
	}


	/**
	 * Convert rgb to hex
	 *
	 * Convert color from rgb value (array) to hex string
	 *
	 * @param (array) rgb - color value as rgb array
	 * 
	 * @return (string) hex - color value as hex string
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function rgb2hex( $rgb ) {
		$hex = array(
			dechex( $rgb['r'] ),
			dechex( $rgb['g'] ),
			dechex( $rgb['b'] )
		);
		return  '#' . ( strlen( $hex[0] ) == 1 ? '0' : '' ) . ( $hex[0] )
					. ( strlen( $hex[1] ) == 1 ? '0' : '' ) . ( $hex[1] )
					. ( strlen( $hex[2] ) == 1 ? '0' : '' ) . ( $hex[2] );
	}


	/**
	 * Convert hsb to hex
	 *
	 * Convert color from hsb value (array) to hex string
	 *
	 * @param (array) hsb - color value as hsb array
	 * 
	 * @return (string) hex - color value as hex string
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function hsb2hex( $hsb ) {
		return $this->rgb2hex( $this->hsb2rgb( $hsb ) );
	}

}
