<?php
namespace TrxEffects\Core\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Messages subsystem.
 *
 * Handling plugin's messages for admin and frontend mode
 *
 * @since 1.0.0
 */
class Messages {

	/**
	 * Options key for the admin mode.
	 *
	 * Options key to store message for the next session in the admin mode.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var admin_key
	 */
	private $admin_key = 'trx_effects_admin_message';

	/**
	 * Message for the admin mode.
	 *
	 * Hold a last message in the admin mode
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var admin_message
	 */
	private $admin_message = array( 'success' => '', 'error' => '' );

	/**
	 * Options key for the frontend mode.
	 *
	 * Options key to store message for the next session in the frontend mode.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var frontend_key
	 */
	private $front_key = 'trx_effects_front_message';

	/**
	 * Message for the frontend mode.
	 *
	 * Hold a last message in the frontend mode
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var frontend_message
	 */
	private $front_message = array( 'success' => '', 'error' => '' );


	/**
	 * Class constructor.
	 *
	 * Initializing options.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/init', array( $this, 'init') );
		add_action( 'wp_footer', array( $this, 'show_front_message') );
	}


	/**
	 * Init messages
	 *
	 * Prepare message from a previous session
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {
		// Admin message
		if ( is_admin() ) {
			if ( ( $msg = get_option( $this->admin_key ) ) != '' ) {
				$this->set_admin_message( $msg );
				update_option( $this->admin_key, '' );
			}

		// Frontend message
		} else {
			if ( ( $msg = get_option( $this->front_key ) ) != '' ) {
				$this->set_front_message( $msg );
				update_option( $this->front_key, '' );
			}
		}
	}


	/**
	 * Set admin message
	 *
	 * Store message to display in the admin mode
	 *
	 * @param (string)         $msg - text of the message
	 * @param (boolean|string) $type - type of the message (success | error)
	 * @param (boolean)        $next_session - show the message on next page load
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function set_admin_message( $msg = false, $type = false, $next_session = false ) {
		if ( $next_session ) {
			$store = array( 'error' => '', 'success' => '' );
			if ( ! empty( $type ) && ! empty( $msg ) ) $store[ $type ] = $msg;
			update_option( $this->admin_key, $store );
		} else if ( ! empty( $msg ) ) {
			if ( empty( $type ) ) {
				$this->admin_message = is_array( $msg ) ? $msg : array( 'error' => '', 'success' => $msg );
			} else {
				$this->admin_message[ $type ] = $msg;
			}
		}
	}


	/**
	 * Get admin message
	 *
	 * Return last message in the admin mode
	 *
	 * @param (boolean|string) $type - type of the message (success | error)
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_admin_message( $type = false ) {
		return empty( $type ) ? $this->admin_message : $this->admin_message[ $type ];
	}


	/**
	 * Set frontend message
	 *
	 * Store message to display in the frontend mode
	 *
	 * @param (string)  $msg - text of the message
	 * @param (boolean) $type - type of the message (success | error)
	 * @param (boolean) $next_session - show the message on next page load
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function set_front_message( $msg = false, $type = false, $next_session = false ) {
		if ( $next_session ) {
			$store = array( 'error' => '', 'success' => '' );
			if ( ! empty( $type ) && ! empty( $msg ) ) $store[ $type ] = $msg;
			update_option( $this->front_key, $store );
		} else if ( ! empty( $msg ) ) {
			if ( empty( $type ) ) {
				$this->front_message = is_array( $msg ) ? $msg : array( 'error' => '', 'success' => $msg );
			} else {
				$this->front_message[ $type ] = $msg;
			}
		}
	}


	/**
	 * Get frontend message
	 *
	 * Return last message in the frontend mode
	 *
	 * @param (boolean|string) $type - type of the message (success | error)
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_front_message( $type = false ) {
		return empty( $type ) ? $this->front_message : $this->front_message[ $type ];
	}


	/**
	 * Show frontend message
	 *
	 * Display message in the frontend mode
	 *
	 * @hook action 'wp_footer'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function show_front_message() {
		$result = $this->get_front_message();
		if ( ! empty( $result['error'] ) ) {
			?><div class="trx_effects_message_box trx_effects_message_box_system trx_effects_message_box_error">
				<h6 class="trx_effects_message_box_title"><?php esc_html_e( 'Error!', 'trx-effects' ); ?></h6>
				<div class="trx_effects_message_box_text"><?php echo wp_kses_data( $result['error'] ); ?></div>
			</div><?php
		} else if ( ! empty( $result['success'] ) ) {
			?><div class="trx_effects_message_box trx_effects_message_box_system trx_effects_message_box_success">
				<h6 class="trx_effects_message_box_title"><?php esc_html_e( 'Success!', 'trx-effects' ); ?></h6>
				<div class="trx_effects_message_box_text"><?php echo wp_kses_data( $result['success'] ); ?></div>
			</div><?php
		}
	}

}
